@@ -5,6 +5,7 @@ try:
     import torch.nn.functional as F
 except:
     print('warning! pytorch not installed')
+    print('警告! 没有安装pytorch，所有pytorch相关函数不可用!')
     class torch():
         Tensor = Exception
 from functools import wraps
@@ -634,7 +635,7 @@ def np_gather_righthand(src, index, check=True):
 
 
 def distance_matrix(A):
-    assert A.shape[-1] == 2  # assert 2D situation
+    # assert A.shape[-1] == 2  # assert 2D situation
     n_subject = A.shape[-2]  # is 2
     A = np.repeat(np.expand_dims(A, -2), n_subject, axis=-2)  # =>(64, 100, 100, 2)
     At = np.swapaxes(A, -2, -3)  # =>(64, 100, 100, 2)
@@ -650,7 +651,7 @@ def delta_matrix(A):
     return delta
 
 def np_normalize_last_dim(mat):
-    return mat / np.expand_dims(np.linalg.norm(mat, axis=2) + 1e-16, axis=-1)
+    return mat / np.expand_dims(np.linalg.norm(mat, axis=-1) + 1e-16, axis=-1)
 
 def dir2rad(delta_pos):
     result = np.empty(delta_pos.shape[:-1], dtype=complex)
@@ -660,6 +661,15 @@ def dir2rad(delta_pos):
     return rad_angle
 
 
+def dir3d_rad(delta_pos):
+    assert delta_pos.shape[-1]==3
+    xy = delta_pos[..., :2]
+    r1 = dir2rad(xy)
+    xy_norm = np.linalg.norm(xy, axis=-1)
+    r2 = dir2rad(np.stack((xy_norm, delta_pos[..., 2]),-1))
+    return np.stack((r1,r2), axis=-1)
+    
+
 def reg_deg(deg):
     return (deg + 180) % 360 - 180
 
